/*  keynames.c: (c) 2002 sibn

    This file is part of GWiz.

    GWiz is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    GWiz is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with GWiz; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <string.h>
#include <stdio.h>
#include "SDL/SDL_keysym.h"
#include "keynames.h"

EnumStringPair keytable[] = {
    { SDLK_UNKNOWN, "Unknown" },
    { SDLK_BACKSPACE, "Backspace" },
    { SDLK_TAB, "Tab" },
    { SDLK_CLEAR, "Clear" },
    { SDLK_RETURN, "Return" },
    { SDLK_PAUSE, "Pause" },
    { SDLK_ESCAPE, "Escape" },
    { SDLK_SPACE, "Space" },
    { SDLK_EXCLAIM, "!" },
    { SDLK_QUOTEDBL, "\"" },
    { SDLK_HASH, "#" },
    { SDLK_DOLLAR, "$" },
    { SDLK_AMPERSAND, "&" },
    { SDLK_QUOTE, "'" },
    { SDLK_LEFTPAREN, "(" },
    { SDLK_RIGHTPAREN, ")" },
    { SDLK_ASTERISK, "*" },
    { SDLK_PLUS, "+" },
    { SDLK_COMMA, "," },
    { SDLK_MINUS, "-" },
    { SDLK_PERIOD, "." },
    { SDLK_SLASH, "/" },
    { SDLK_0, "0" },
    { SDLK_1, "1" },
    { SDLK_2, "2" },
    { SDLK_3, "3" },
    { SDLK_4, "4" },
    { SDLK_5, "5" },
    { SDLK_6, "6" },
    { SDLK_7, "7" },
    { SDLK_8, "8" },
    { SDLK_9, "9" },
    { SDLK_COLON, ":" },
    { SDLK_SEMICOLON, ";" },
    { SDLK_LESS, "<" },
    { SDLK_EQUALS, "=" },
    { SDLK_GREATER, ">" },
    { SDLK_QUESTION, "?" },
    { SDLK_AT, "@" },
    { SDLK_a-32, "A" },
    { SDLK_b-32, "B" },
    { SDLK_c-32, "C" },
    { SDLK_d-32, "D" },
    { SDLK_e-32, "E" },
    { SDLK_f-32, "F" },
    { SDLK_g-32, "G" },
    { SDLK_h-32, "H" },
    { SDLK_i-32, "I" },
    { SDLK_j-32, "J" },
    { SDLK_k-32, "K" },
    { SDLK_l-32, "L" },
    { SDLK_m-32, "M" },
    { SDLK_n-32, "N" },
    { SDLK_o-32, "O" },
    { SDLK_p-32, "P" },
    { SDLK_q-32, "Q" },
    { SDLK_r-32, "R" },
    { SDLK_s-32, "S" },
    { SDLK_t-32, "T" },
    { SDLK_u-32, "U" },
    { SDLK_v-32, "V" },
    { SDLK_w-32, "W" },
    { SDLK_x-32, "X" },
    { SDLK_y-32, "Y" },
    { SDLK_z-32, "Z" },
    { SDLK_LEFTBRACKET, "[" },
    { SDLK_BACKSLASH, "\\" },
    { SDLK_RIGHTBRACKET, "]" },
    { SDLK_CARET, "^" },
    { SDLK_UNDERSCORE, "_" },
    { SDLK_BACKQUOTE, "`" },
    { SDLK_a, "a" },
    { SDLK_b, "b" },
    { SDLK_c, "c" },
    { SDLK_d, "d" },
    { SDLK_e, "e" },
    { SDLK_f, "f" },
    { SDLK_g, "g" },
    { SDLK_h, "h" },
    { SDLK_i, "i" },
    { SDLK_j, "j" },
    { SDLK_k, "k" },
    { SDLK_l, "l" },
    { SDLK_m, "m" },
    { SDLK_n, "n" },
    { SDLK_o, "o" },
    { SDLK_p, "p" },
    { SDLK_q, "q" },
    { SDLK_r, "r" },
    { SDLK_s, "s" },
    { SDLK_t, "t" },
    { SDLK_u, "u" },
    { SDLK_v, "v" },
    { SDLK_w, "w" },
    { SDLK_x, "x" },
    { SDLK_y, "y" },
    { SDLK_z, "z" },
    { SDLK_DELETE, "Delete" },
    { SDLK_WORLD_0, "World0" },
    { SDLK_WORLD_1, "World1" },
    { SDLK_WORLD_2, "World2" },
    { SDLK_WORLD_3, "World3" },
    { SDLK_WORLD_4, "World4" },
    { SDLK_WORLD_5, "World5" },
    { SDLK_WORLD_6, "World6" },
    { SDLK_WORLD_7, "World7" },
    { SDLK_WORLD_8, "World8" },
    { SDLK_WORLD_9, "World9" },
    { SDLK_WORLD_10, "World10" },
    { SDLK_WORLD_11, "World11" },
    { SDLK_WORLD_12, "World12" },
    { SDLK_WORLD_13, "World13" },
    { SDLK_WORLD_14, "World14" },
    { SDLK_WORLD_15, "World15" },
    { SDLK_WORLD_16, "World16" },
    { SDLK_WORLD_17, "World17" },
    { SDLK_WORLD_18, "World18" },
    { SDLK_WORLD_19, "World19" },
    { SDLK_WORLD_20, "World20" },
    { SDLK_WORLD_21, "World21" },
    { SDLK_WORLD_22, "World22" },
    { SDLK_WORLD_23, "World23" },
    { SDLK_WORLD_24, "World24" },
    { SDLK_WORLD_25, "World25" },
    { SDLK_WORLD_26, "World26" },
    { SDLK_WORLD_27, "World27" },
    { SDLK_WORLD_28, "World28" },
    { SDLK_WORLD_29, "World29" },
    { SDLK_WORLD_30, "World30" },
    { SDLK_WORLD_31, "World31" },
    { SDLK_WORLD_32, "World32" },
    { SDLK_WORLD_33, "World33" },
    { SDLK_WORLD_34, "World34" },
    { SDLK_WORLD_35, "World35" },
    { SDLK_WORLD_36, "World36" },
    { SDLK_WORLD_37, "World37" },
    { SDLK_WORLD_38, "World38" },
    { SDLK_WORLD_39, "World39" },
    { SDLK_WORLD_40, "World40" },
    { SDLK_WORLD_41, "World41" },
    { SDLK_WORLD_42, "World42" },
    { SDLK_WORLD_43, "World43" },
    { SDLK_WORLD_44, "World44" },
    { SDLK_WORLD_45, "World45" },
    { SDLK_WORLD_46, "World46" },
    { SDLK_WORLD_47, "World47" },
    { SDLK_WORLD_48, "World48" },
    { SDLK_WORLD_49, "World49" },
    { SDLK_WORLD_50, "World50" },
    { SDLK_WORLD_51, "World51" },
    { SDLK_WORLD_52, "World52" },
    { SDLK_WORLD_53, "World53" },
    { SDLK_WORLD_54, "World54" },
    { SDLK_WORLD_55, "World55" },
    { SDLK_WORLD_56, "World56" },
    { SDLK_WORLD_57, "World57" },
    { SDLK_WORLD_58, "World58" },
    { SDLK_WORLD_59, "World59" },
    { SDLK_WORLD_60, "World60" },
    { SDLK_WORLD_61, "World61" },
    { SDLK_WORLD_62, "World62" },
    { SDLK_WORLD_63, "World63" },
    { SDLK_WORLD_64, "World64" },
    { SDLK_WORLD_65, "World65" },
    { SDLK_WORLD_66, "World66" },
    { SDLK_WORLD_67, "World67" },
    { SDLK_WORLD_68, "World68" },
    { SDLK_WORLD_69, "World69" },
    { SDLK_WORLD_70, "World70" },
    { SDLK_WORLD_71, "World71" },
    { SDLK_WORLD_72, "World72" },
    { SDLK_WORLD_73, "World73" },
    { SDLK_WORLD_74, "World74" },
    { SDLK_WORLD_75, "World75" },
    { SDLK_WORLD_76, "World76" },
    { SDLK_WORLD_77, "World77" },
    { SDLK_WORLD_78, "World78" },
    { SDLK_WORLD_79, "World79" },
    { SDLK_WORLD_80, "World80" },
    { SDLK_WORLD_81, "World81" },
    { SDLK_WORLD_82, "World82" },
    { SDLK_WORLD_83, "World83" },
    { SDLK_WORLD_84, "World84" },
    { SDLK_WORLD_85, "World85" },
    { SDLK_WORLD_86, "World86" },
    { SDLK_WORLD_87, "World87" },
    { SDLK_WORLD_88, "World88" },
    { SDLK_WORLD_89, "World89" },
    { SDLK_WORLD_90, "World90" },
    { SDLK_WORLD_91, "World91" },
    { SDLK_WORLD_92, "World92" },
    { SDLK_WORLD_93, "World93" },
    { SDLK_WORLD_94, "World94" },
    { SDLK_WORLD_95, "World95" },
    { SDLK_KP0, "Num 0" },
    { SDLK_KP1, "Num 1" },
    { SDLK_KP2, "Num 2" },
    { SDLK_KP3, "Num 3" },
    { SDLK_KP4, "Num 4" },
    { SDLK_KP5, "Num 5" },
    { SDLK_KP6, "Num 6" },
    { SDLK_KP7, "Num 7" },
    { SDLK_KP8, "Num 8" },
    { SDLK_KP9, "Num 9" },
    { SDLK_KP_PERIOD, "Num ." },
    { SDLK_KP_DIVIDE, "Nun /" },
    { SDLK_KP_MULTIPLY, "Num *" },
    { SDLK_KP_MINUS, "Num -" },
    { SDLK_KP_PLUS, "Num +" },
    { SDLK_KP_ENTER, "Num Ret" },
    { SDLK_KP_EQUALS, "Num =" },
    { SDLK_UP, "Up" },
    { SDLK_DOWN, "Down" },
    { SDLK_RIGHT, "Right" },
    { SDLK_LEFT, "Left" },
    { SDLK_INSERT, "Insert" },
    { SDLK_HOME, "Home" },
    { SDLK_END, "End" },
    { SDLK_PAGEUP, "PgUp" },
    { SDLK_PAGEDOWN, "PgDown" },
    { SDLK_F1, "F1" },
    { SDLK_F2, "F2" },
    { SDLK_F3, "F3" },
    { SDLK_F4, "F4" },
    { SDLK_F5, "F5" },
    { SDLK_F6, "F6" },
    { SDLK_F7, "F7" },
    { SDLK_F8, "F8" },
    { SDLK_F9, "F9" },
    { SDLK_F10, "F10" },
    { SDLK_F11, "F11" },
    { SDLK_F12, "F12" },
    { SDLK_F13, "F13" },
    { SDLK_F14, "F14" },
    { SDLK_F15, "F15" },
    { SDLK_NUMLOCK, "Numlock" },
    { SDLK_CAPSLOCK, "Capslock" },
    { SDLK_SCROLLOCK, "Scrollock" },
    { SDLK_RSHIFT, "Rshift" },
    { SDLK_LSHIFT, "Lshift" },
    { SDLK_RCTRL, "Rctrl" },
    { SDLK_LCTRL, "Lctrl" },
    { SDLK_RALT, "Ralt" },
    { SDLK_LALT, "Lalt" },
    { SDLK_RMETA, "Rmeta" },
    { SDLK_LMETA, "Lmeta" },
    { SDLK_LSUPER, "Lsuper" },
    { SDLK_RSUPER, "Rsuper" },
    { SDLK_MODE, "Mode" },
    { SDLK_COMPOSE, "Compose" },
    { SDLK_HELP, "Help" },
    { SDLK_PRINT, "Print" },
    { SDLK_SYSREQ, "Sysrq" },
    { SDLK_BREAK, "Break" },
    { SDLK_MENU, "Menu" },
    { SDLK_POWER, "Power" },
    { SDLK_EURO, "Euro" },
    { SDLK_UNDO, "Undo" },
    { 0, NULL }
};

char *KeyToChar (int key)
{
    EnumStringPair *k = keytable;
    int which = 0;

    while (k[which].name)
	{
	    if (k[which].num == key)
		return (k[which].name);
	    which++;
	}
    return NULL;
}

int CharToKey (char *key, int defval)
{
    EnumStringPair *k = keytable;
    int which = 0;

    while (k[which].name)
	{
	    if (strcasecmp (k[which].name, key) == 0)
		return (k[which].num);
	    which++;
	}
    return (defval);
}
